% SUM-MAX MAPS FOR TEMPLATE MATCHING INFERENCE 
clear all; close all; 
load working/parameter; 
load working/testingImage; 
%% prepare variables for SUM-MAX maps
allSizex = zeros(1, numResolution); allSizey = zeros(1, numResolution); 
MAX1maps = cell(numResolution, numOrient); 
trackMap = cell(numResolution, numOrient);
SUM2map = cell(1, numResolution); 
for(img=1:numResolution)
    [sizex, sizey] = size(J{img}); 
    allSizex(img) = sizex; allSizey(img) = sizey; % sizes of images at multiple resolutions
    sizexSubsample = floor(sizex/subsample); sizeySubsample = floor(sizey/subsample); 
    SUM2map{img} = single(zeros(floor((sizex+sizeTemplatex)/subsample), floor((sizey+sizeTemplatey)/subsample)));  % log-likelihood score maps for testing images
    averageMap{img} = single(zeros(sizex, sizey));
    for (orient=1:numOrient)
        MAX1map{img, orient} = single(zeros(sizexSubsample, sizeySubsample)); 
        trackMap{img, orient} = single(zeros(sizexSubsample, sizeySubsample));
    end
    translatedTemplate{img} = single(zeros(sizex, sizey));  % symbolic sketch on testing image
end
MAX2score = single(zeros(1, numResolution));  % maximum log-likelihood score at each resolution
Fx = zeros(1, numResolution); Fy = zeros(1, numResolution); % maximum likelihood location at each resolution
%% find over multiple scales
for (s = 1 : numScale)
  storeName = ['working/storedExponentialModel' num2str(s)];   
  load(storeName); 
  templateName = ['working/learnedTemplate' num2str(s)];   
  load(templateName); 
%%  compute SUM1 maps for all resolutions
disp(['start filtering']); tic
SUM1map = ApplyFilterfft(J, allFilter, localOrNot, localHalfx, localHalfy);  % filter testing image at all resolutions
disp(['filtering time: ' num2str(toc) ' seconds']);
%% compute the averageMap
Caverage(numResolution, allSizex, allSizey, numOrient, halfFilterSize, SUM1map, averageMap, templateHalfx, templateHalfy, windowNormalizeOrNot);
%% compute MAX1 maps for all resolutions
disp(['start maxing']); tic
CgetMAX1track(numResolution, allSizex, allSizey, numOrient, locationShiftLimit, orientShiftLimit, subsample, ...
              SUM1map, MAX1map, trackMap); 
disp(['maxing time: ' num2str(toc) ' seconds']);
mapName = ['working/SUMMAXmap' num2str(s)];   
save(mapName, 'SUM1map', 'MAX1map', 'trackMap', 'averageMap'); 
%% compute and accumulate the SUM2 maps
disp(['start finding']); tic
CgetSUM2(numResolution, allSizex, allSizey, numOrient, subsample, ...
         sizeTemplatex, sizeTemplatey, numElement, selectedOrient, selectedx, selectedy, selectedlambda, selectedLogZ, ...
         MAX1map, SUM2map, averageMap, saturation, thresholdFactor);
disp(['finding time: ' num2str(toc) ' seconds']);
end
%% compute MAX2 scores for all resolutions
CgetMAX2(numResolution, allSizex, allSizey, subsample, ...
         sizeTemplatex, sizeTemplatey, SUM2map, MAX2score, Fx, Fy);
%% if inhibit, recalculate MAX2 score for each resolution
if (inhibitFind>0.)
    MAX2score = single(zeros(1, numResolution));  % recalculate MAX2 score at each resolution
   for (s = 1 : numScale)
      storeName = ['working/storedExponentialModel' num2str(s)];   
      load(storeName); 
      templateName = ['working/learnedTemplate' num2str(s)];   
      load(templateName); 
      mapName = ['working/SUMMAXmap' num2str(s)];   
      load(mapName); 
      CgetBackInhibit(numResolution, allSizex, allSizey, numOrient, ...
         locationShiftLimit, orientShiftLimit, subsample, halfFilterSize, allSymbol(1, :), ...
         numElement, selectedOrient, selectedx, selectedy, selectedlambda, selectedLogZ, ...
         SUM1map, MAX1map, trackMap, translatedTemplate, Fx, Fy, saturation, ...
         sizeTemplatex, sizeTemplatey, Correlation, MAX2score, averageMap);
   end
end
%% find the optimal resolution
[maxOverResolution ind] = max(MAX2score);   % most likely resolution
Outepsgif(J{ind},  ['output/cartest' num2str(ii) 'original' num2str(s)]); % original image 
%% plot SUM2 map and MAX2 scores
figure('Position', [1 1 1024 768]); 
surf(double(SUM2map{ind})); % log-likelihood score map at optimal resolution
set(gca,'FontSize',18); % adjust the display format and save the plot
set(gcf, 'PaperPositionMode', 'auto');
print('-depsc', ['output/car' num2str(ii) 'SUM2' num2str(s) '.eps']);
figure; h1=plot(MAX2score); 
title('MAX2 score vs scale'); xlabel('scale'); ylabel('MAX2');
set(gca,'FontSize',16); % adjust the display format and save the plot
set(get(gca,'XLabel'),'FontSize',18);
set(get(gca,'YLabel'),'FontSize',18);
set(get(gca,'Title'),'FontSize',20);
set(h1,'Linewidth',3);
set(gcf, 'PaperPositionMode', 'auto');
print('-depsc', ['output/car' num2str(ii) 'MAX2' num2str(s) '.eps']);
%% superpose templates at multiple scales
if (inhibitFind>0.)
    MAX2score = single(zeros(1, numResolution));  % recalculate MAX2 score at each resolution
end
for (s = 1 : numScale)
      storeName = ['working/storedExponentialModel' num2str(s)];   
      load(storeName); 
      templateName = ['working/learnedTemplate' num2str(s)];   
      load(templateName); 
      mapName = ['working/SUMMAXmap' num2str(s)];   
      load(mapName); 
      if (inhibitFind>0.)
         CgetBackInhibit(numResolution, allSizex, allSizey, numOrient, ...
         locationShiftLimit, orientShiftLimit, subsample, halfFilterSize, allSymbol(1, :), ...
         numElement, selectedOrient, selectedx, selectedy, selectedlambda, selectedLogZ, ...
         SUM1map, MAX1map, trackMap, translatedTemplate, Fx, Fy, saturation, ...
         sizeTemplatex, sizeTemplatey, Correlation, MAX2score, averageMap);
      else
         CgetBack(numResolution, allSizex, allSizey, numOrient, ...
         locationShiftLimit, orientShiftLimit, subsample, halfFilterSize, allSymbol(1, :), ...
         numElement, selectedOrient, selectedx, selectedy, ...
         MAX1map, trackMap, translatedTemplate, Fx, Fy, saturation);
      end
      Outepsgif(J{ind}+translatedTemplate{ind}*100, ['output/car' num2str(ii) 'find' num2str(s)]); % overlaid sketch at optimal resolution  
end


